%% Solutions for Practical Exercise 3 of the ICN Matlab Course 2016
%  Daniel Bush, UCL (2016) d.bush@ucl.ac.uk

% Part One: Import the SeqMemData.mat file into Matlab. This contains
% sequence and source memory accuracy data for two conditions (successive
% objects in the sequence either being located in the same room or a
% different room) from 38 participants that each completed two experimental
% blocks, as well as the average time between viewing successive objects in
% the sequence for each participant and condition. First, average the
% SequenceMemory, SourceMemory and AverageTime values across blocks for
% each condition (the condition labels are provided in Key) to obtain
% mean performance values for each participant in each condition. Next,
% assess whether there is any significant difference in mean sequence
% memory accuracy between conditions across participants, and in mean
% source memory accuracy between conditions across participants. Next,
% assess whether there is a significant correlation between average time
% and sequence memory accuracy, and between average time and source memory
% within each condition across participants. Finally, assess whether there
% is a significant correlation between sequence and source memory scores in
% each condition across participants. Save the t-statistics and p-values
% from each analysis, as well as the averaged data, in a *.mat file named
% SeqMemAnalysis.mat

% First, load the Matlab data file into memory. To do this, make sure it is
% in your current folder by changing your current folder at the top of the
% main Matlab window, then use the 'load' function:
load('SeqMemData.mat');                                 % Load the data file

% The 'Key' variable tells us that the accuracy scores for the 'same'
% condition from blocks 1 and 2 are located in columns 1 and 3, while the
% scores for the 'different' condition from blocks 1 and 2 are located in
% columns 2 and 4. Hence, we will first compute the mean scores for each
% participant across blocks by taking the average of these pairs of
% columns. Note that, by default, the 'mean' function generates the mean of
% each column of input data, so we will also need to provide a second input
% to specify the dimension we would like the mean taken across, which is 
% the second dimension (i.e. each row) in this case, and hence takes a 
% value of 2:

Seq.SameAcc     = mean(SequenceMemory(:,[1 3]), 2);     % Compute mean sequence memory accuracy for each participant in the 'same' condition across blocks 
Seq.DiffAcc     = mean(SequenceMemory(:,[2 4]), 2);     % Compute mean sequence memory accuracy for each participant in the 'different' condition across blocks

Src.SameAcc 	= mean(SourceMemory(:,[1 3]), 2);       % Compute mean source memory accuracy for each participant in the 'same' condition across blocks
Src.DiffAcc     = mean(SourceMemory(:,[2 4]), 2);       % Compute mean source memory accuracy for each participant in the 'different' condition across blocks

Time.Same       = mean(AverageTime(:,[1 3]), 2);        % Compute the mean time between objects in the 'same' condition across blocks
Time.Diff       = mean(AverageTime(:,[2 4]), 2);        % Compute the mean time between objects in the 'different' condition across blocks

% Next, to see if there are any significant differences in accuracy scores
% between these condition, we use the Matlab 'ttest' function, which
% performs a one-sample (or paired) t-test on the data provided as input:

[Seq.h Seq.p]  	= ttest(Seq.SameAcc, Seq.DiffAcc);      % Compare mean sequence memory accuracy between 'same' and 'different' conditions
[Src.h Src.p]   = ttest(Src.SameAcc, Src.DiffAcc);      % Compare mean source memory accuracy between 'same' and 'different' conditions

% Next, to see if there is a significant correlation between the average
% time taken to move between objects and sequence or source memory in each
% condition, we make use of the Matlab 'corrcoef' function. This provides a
% matrix of correlation coefficients and p values - those in the first row,
% second column and second row, first column should be identical, and give
% the correlation coefficient and p-value for the relationship between the
% two input arrays, which is what we are interested in. We can therefore 
% store this value alone, rather than the full matrix:

[Seq.same_r Seq.same_p] = corrcoef(Time.Same, Seq.SameAcc);     % Compute the correlation co-efficients for time v sequence memory in the 'same' condition
Seq.same_r              = Seq.same_r(1,2);                      % Extract the correlation co-efficient of interest
Seq.same_p              = Seq.same_p(1,2);                      % Extract the p value of interest
[Seq.diff_r Seq.diff_p] = corrcoef(Time.Diff, Seq.DiffAcc);     % Compute the correlation co-efficients for time v sequence memory in the 'differnt' condition
Seq.diff_r              = Seq.diff_r(1,2);                      % Extract the correlation co-efficient of interest
Seq.diff_p              = Seq.diff_p(1,2);                      % Extract the p value of interest

[Src.same_r Src.same_p] = corrcoef(Time.Same, Src.SameAcc);     % Compute the correlation co-efficients for time v source memory in the 'same' condition
Src.same_r              = Src.same_r(1,2);                      % Extract the correlation co-efficient of interest
Src.same_p              = Src.same_p(1,2);                      % Extract the p value of interest
[Src.diff_r Src.diff_p] = corrcoef(Time.Diff, Src.DiffAcc);     % Compute the correlation co-efficients for time v source memory in the 'different' condition
Src.diff_r              = Src.diff_r(1,2);                      % Extract the correlation co-efficient of interest
Src.diff_p              = Src.diff_p(1,2);                      % Extract the p value of interest

% Next, we need to run a couple more correlation analyses to see if there
% is any relationship between sequence and source memory scores within each
% condition. This is performed in exactly the same way:

[SeqSrc.same_r SeqSrc.same_p]   = corrcoef(Seq.SameAcc, Src.SameAcc);       % Compute the correlation co-efficients for sequence v source memory in the 'same' condition
SeqSrc.same_r           = SeqSrc.same_r(1,2);                               % Extract the correlation co-efficient of interest
SeqSrc.same_p           = SeqSrc.same_p(1,2);                               % Extract the p value of interest
[SeqSrc.diff_r SeqSrc.diff_p]   = corrcoef(Seq.DiffAcc, Src.DiffAcc);       % Compute the correlation co-efficients for sequence v source memory in the 'different' condition
SeqSrc.diff_r           = SeqSrc.diff_r(1,2);                               % Extract the correlation co-efficient of interest
SeqSrc.diff_p           = SeqSrc.diff_p(1,2);                               % Extract the p value of interest

% Finally, we need to save the output of these analyses using the Matlab
% 'save' function - providing, as an input, the file name and (optionally),
% a list of all the variables we would like to save. If we just provide a
% file name, then Matlab will save all of the variables in the workspace. 

save('SeqMemAnalysis.mat','Seq','Src','Time','SeqSrc');                     % Save the analysis results in a *.mat file



%% Part Two: Import all information from the Data1.xlsx file into Matlab.
% Compute the mean and standard error of accuracy and RT values for each
% condition across all participants. Assess whether there is any
% significant difference in accuracy and reaction times between conditions
% across all participants. Assess whether there is any difference in mean
% accuracy between participants that are left- and right- handed. Compute
% the correlation coefficient and p value for the relationship between age
% and mean accuracy across both conditions for each participant. Save the
% mean and standard error of accuracy and RT values for each condition, as
% well as the results of all statistical analyses, in a *.mat file named
% Data1Analysis.mat

% First, we need to import data from the Excel spreadsheet into memory. We
% do this using the xlsread function (note that this will only work on a
% PC, as this feature is - unfortunately - no longer supported by Mac OS).
% Again, make sure that the data file is in your current folder by changing
% your current folder in the main Matlab window. 

[num txt raw] = xlsread('Data1.xlsx');                  % Load the data from an Excel file

% An inspection of the 'txt' variable, via the Matlab variable editor
% (accessed by double clicking on the variable name in the workspace
% window), shows us that columns 5 - 8 contain the accuracy and reaction
% time (RT) values for each participant in two conditions. To compute the
% mean and standard error, therefore, we simply take the mean and standard
% deviation of those columns in 'num', dividing the latter by the number of
% samples (i.e. the number of rows) to get the standard error. We can
% automatically compute the number of rows using the 'size' function:

Cond1.MeanAcc   = mean(num(:,5));                       % Compute overall mean accuracy in condition 1
Cond1.MeanRT    = mean(num(:,6));                       % Compute overall mean RT in condition 1
Cond1.StdEAcc   = std(num(:,5)) ./ sqrt(size(num,1));   % Compute the standard error of accuracy in condition 1
Cond1.StdERT    = std(num(:,6)) ./ sqrt(size(num,1));   % Compute the standard error of RTs in condition 1

Cond2.MeanAcc   = mean(num(:,7));                       % Compute overall mean accuracy in condition 2
Cond2.MeanRT    = mean(num(:,8));                       % Compute overall mean RT in condition 2
Cond2.StdEAcc   = std(num(:,7)) ./ sqrt(size(num,1));   % Compute the standard error of accuracy in condition 2
Cond2.StdERT    = std(num(:,8)) ./ sqrt(size(num,1));   % Compute the standard error of RTs in condition 2

% As above, we can use the 'ttest' function to assess whether there is any
% significant difference in accuracy and reaction times between conditions:

[Acc_h Acc_p]   = ttest(num(:,5),num(:,7));             % Compare mean accuracy in conditions 1 and 2
[RT_h  RT_p]    = ttest(num(:,6),num(:,8));             % Compare mean RT in conditions 1 and 2

% Next, to compare mean accuracy between left- and right- handed 
% participants, we first need to compute the mean accuracy across 
% conditions: 

MeanAcc         = mean(num(:,[5 7]),2);                 % Extract mean accuracy across both conditions

% Next, we need to use a two-sample t-test, which is performed with the Matlab
% 'ttest2' function, using the same syntax. The only tricky bit here is
% obtaining the row indices for left- and right- handed participants. The
% most straightforward method of doing this is to use the 'strmatch'
% function on the data stored in the second column of either 'raw' or
% 'txt', which is where the handedness data is stored, making sure to
% ignore the first row (where the header information is stored). The
% 'strmatch' function returns the indices of the second input that match
% the string provided as the first input, i.e. they will tell us which rows
% of the 'Handedness' column are equal to 'L' and 'R' respectively:

lefties         = strmatch('L', raw(2:25,2));                   % Extract row indices of left-handed participants
righties        = strmatch('R', raw(2:25,2));                   % Extract row indices of right-handed participants
[hand_h hand_p] = ttest2(MeanAcc(lefties), MeanAcc(righties));  % Compare mean accuracy across conditions between left- and right- handed participants
clear lefties righties

% Next, we can use the 'corrcoef' function again, to see if there is a
% relationship between age (which is stored in the third column of 'num') 
% and mean accuracy across conditions:

[age_r age_p]   = corrcoef(num(:,3), MeanAcc);                  % Compute the correlation co-efficients for age vs mean accuracy across conditions
age_r           = age_r(1,2);                                   % Extract the correlation co-efficient of interest
age_p           = age_p(1,2);                                   % Extract the p value of interest

% Finally, we can save the results of all analyses as described above:

save('Data1Analysis.mat','Cond1','Cond2','Acc_h','Acc_p','RT_h','RT_p','hand_h','hand_p','age_r','age_p');